/*************************************************************************/
/*
//  Vehicle Routing Problem With GA
//
//
*/
/*************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <new.h>

#include "VRPFileReader.h"
#include "VRPSolution.h"
#include "TestDefinition.h"
#include "EdgeGraph.h"

/*#include "AirScheduleProblem.h"
#include "AirScheduleSolu2.h"
#include "AirFitnessFunction.h"
#include "AirInitializer.h"
#include "AirMutation.h"
#include "AirCrossover.h"
//#include "AirCrossover2.h"
#include "City.h"
#include <deffunc.h>

#include <Sort_Recombinator.h>
#include <Stat_Recombinator.h>
#include <Generational_Recom.h>

#include <SimilarParentSelector.h>
#include <GeneticAlgorithm.h>

typedef long time_t;
typedef long clock_t;
*/

/*
// Memory Allocation Error Handler.
// It will be called when there is no Enough 
// Memory.
*/

void allocation_handler()
{
   cout << "Out of Memory. Aborting Program...."<< endl;
   exit(1);
}

/**
* This part tries to analyze the effect of random initialization part.
*/
mainReadProblem(char *cfileName)
{
   int i; 

   // Set Allocation Exception Handler
   set_new_handler(allocation_handler);
   srand(150);

   TestDefinition tst ;
   tst = tst.readTestDefinition(cfileName);

   /** Read the input data */  
   VRPFileReader Reader(tst.getDataFileName());  
   VRProblem Prob;
   Reader.read(Prob);

   gVector<customerId> depots = tst.getDepots();
   customerId *id;
   for(i=0;i<depots.getSize();i++) 
      if((id=depots.getNthElement(i))!=NULL) Prob.addDepot(*id);

   cout << Prob << endl;

   DepotList::println(Prob.getDepotList());
   CustomerList::println(Prob.getCustomerList());

   EdgeGraph graph(&Prob,Prob.getNumberOfCustomers());
   graph.buildGraphFor(&Prob);
   Prob.setEdgeGraph(&graph);

   cout << graph << endl;
   VRPSolution aSolution(&Prob);
   aSolution.solveRandom();
   cout << aSolution << endl;
   cout << Prob << endl;

   ofstream out("Distance.out");
   out << "File Name: "<< tst.getDataFileName() << endl;

   int cendl=4,pp=0,numofnodes=Prob.getNumberOfCustomers();
   Customer aCustomer;

   out << setw(80) << setfill('*') << " " << endl ;
   out << setfill(' ');
   for(i=0;i<numofnodes;i++)  
     if(graph.getNode(i,aCustomer)) 
	out << " Node[" << setw(3) << i << "]: " << aCustomer ;
   out << setw(80) << setfill('*') << " " << endl; 
   out << setfill(' ');

   for(i=0;i<numofnodes;i++)  {
     pp=0; cendl=4; 
     for(int j=0;j<numofnodes;j++) 
       if(graph.doesEdgeExist(i,j)) { 
	 out << " d(" << setw(3) << i << "," 
		      << setw(3) << j << ") " 
		      << setw(7) << graph.getCost(i,j) ; 
         cendl--; pp=1;
         if(cendl==0) { out<<endl; cendl=4; }
       }// end of if
     if(cendl<4&&pp) out<<endl;
   }// end of for

   out.close();
   
   return 1;

}// end of mainReadProblem()

main(int argc, char *argv[])
{

   if(argc>0) mainReadProblem(argv[1]);
   else
   {
      cout << " usage :  printProblem <testdefinitionfilename> " << endl;
      cout << " test definition file name describes the current test " << endl;
      cout << " parameters. " << endl << endl << endl;
   }

}// end of main

